# -*- coding: utf-8 -*-
from django.template import Variable, TemplateSyntaxError, VariableDoesNotExist

def resolve_string_or_variable(value, context, safe=False, default=None, capture_none_value=True):
    """
    Resolve a template variable within the given context
    
    If the variable is a string (detected with surrounded quotes or double-quotes) it is 
    directly returned, else the ``Variable(value).resolve(context)`` method will be used 
    to find the variable in the context.
    
    As opposite to ``Variable().resolve()`` this method can silently work with ``None`` or 
    empty value.
    
    The process is :
    
    * Surrounded variable by single ou double quotes mean a string that is returned without 
      the surround quotes;
    * Else try to resolve it in the context;
    * If the resolve fail, raise an exception ``TemplateSyntaxError`` if ``safe`` is ``False`` 
      else return the ``default`` value that is ``None`` by default.
    
    :type value: string
    :param value: Valeur à résoudre. C'est soit directement une chaîne de caractères, 
                  soit un nom de variable à résoudre dans le context du template.
    
    :type context: object `django.template.RequestContext`
    :param context: Objet du contexte du template en cours ou sera recherché la valeur 
                    de la variable si ce n'est pas un simple string.
    
    :type safe: bool
    :param safe: (optional) Indique que l'échec pour retrouver le contenu de la variable 
                 doit être silencieux. Si activé, la valeur par défaut de ``default`` 
                 est renvoyée. False par défaut, l'échec remonte une exception 
                 `django.template.TemplateSyntaxError`.
    
    :type default: any
    :param default: (optional) Valeur par défaut renvoyée en cas d'échec pour retrouver 
                    la variable et si le mode silencieux est activé. Par défaut la 
                    valeur est ``None``.
    
    :type capture_none_value: bool
    :param capture_none_value: (optional) Indique si l'on doit capturer une variable du 
                               nom de "None" directement comme un ``None`` sans tenter de 
                               la résoudre comme un nom de variable. Dans ce cas, "None" 
                               deviendra un nom réservé pour le templatetag en cours. Si 
                               cette argument vaut ``False``, la méthode essayera de 
                               résoudre "None" comme un nom de variable dans le contexte. 
                               Par défaut cette option est activé (``True``).
    
    :rtype: any
    :return: La valeur de la variable si retrouvée, sinon la valeur par défaut.
    """
    # Capture du nom de variable "None" directement comme un ``None``
    if capture_none_value and value == "None":
        return None
    
    if value != None:
        # Ne résoud pas les chaînes de texte (toujours encerclés par des simples ou 
        # doubles quotes)
        if not(value[0] == value[-1] and value[0] in ('"', "'")):
            # Tente de résoudre le nom de variable dans le context donné
            try:
                value = Variable(value).resolve(context)
            except VariableDoesNotExist:
                # Échec qui remonte une exception de template
                if not safe:
                    raise TemplateSyntaxError("Unable to resolve '%s'" % value)
                # Échec silencieux qui renvoi la valeur par défaut sans lever d'exception
                else:
                    return default
        # Chaîne de texte dont on retire les quotes d'encerclement de la valeur
        else:
            value = value[1:-1]
    
    return value



# -*- coding: utf-8 -*-
"""
Data models
"""
from datetime import datetime

import django.dispatch
from django.db import models
from django.contrib.auth.models import User
from django.core.cache import cache
from django.utils.translation import ugettext_lazy as _

import mptt
from mptt.models import TreeForeignKey

from rstview.local_settings import RSTVIEW_PARSER_FILTER_SETTINGS

from sveedocuments.local_settings import (DOCUMENTS_PAGE_TEMPLATES, PAGE_SLUGS_CACHE_KEY_NAME, 
                                        PAGE_RENDER_CACHE_KEY_NAME, INSERT_RENDER_CACHE_KEY_NAME,
                                        PAGE_TOC_CACHE_KEY_NAME, INSERT_TOC_CACHE_KEY_NAME,
                                        DOCUMENTS_PAGE_ARCHIVED, DOCUMENTS_PAGE_TEMPLATE_DEFAULT)
from sveedocuments.utils import _get_cache_keyset
from sveedocuments.utils.filefield import content_file_name

DOCUMENTS_PAGE_TEMPLATES_CHOICES = [(k,v[1]) for k,v in DOCUMENTS_PAGE_TEMPLATES.items()]

DOCUMENTS_VISIBILTY_CHOICES = (
    (True, _('Visible')),
    (False, _('Hidden')),
)

IMAGE_MIMETYPES = (
    'image/gif',
    'image/jpeg',
    'image/png',
    #'image/tiff',
    'image/svg+xml',
)

ATTACH_FILE_UPLOADTO = lambda x,y: content_file_name('pages/attachments/%Y/%m/%d', x, y)

class Insert(models.Model):
    """
    Document to insert
    """
    created = models.DateTimeField(_('created'), blank=True, auto_now_add=True)
    modified = models.DateTimeField(_('last edit'), auto_now=True)
    author = models.ForeignKey(User, verbose_name=_('author'))
    title = models.CharField(_('title'), blank=True, null=True, max_length=255)
    slug = models.SlugField(_('slug'), unique=True, max_length=75)
    visible = models.BooleanField(_('visibility'), choices=DOCUMENTS_VISIBILTY_CHOICES, default=True)
    content = models.TextField(_('content'), blank=False)

    def __unicode__(self):
        return self.slug
    
    def get_render_cache_key(self, **kwargs):
        """
        Get the cache key for the content render according to the given settings
        """
        return INSERT_RENDER_CACHE_KEY_NAME.format(id=self.id, **kwargs)
    
    def get_toc_cache_key(self, **kwargs):
        """
        Get the cache key for the content TOC according to the given settings
        """
        return INSERT_TOC_CACHE_KEY_NAME.format(id=self.id, **kwargs)
    
    def save(self, *args, **kwargs):
        # Invalidate all caches at edit
        if self.modified:
            self.clear_cache()
        
        super(Insert, self).save(*args, **kwargs)
    
    def delete(self, using=None):
        self.clear_cache()
        super(Insert, self).delete(using=using)
    
    def clear_cache(self):
        """
        Invalidate all possible cache keys
        """
        keys = _get_cache_keyset(INSERT_RENDER_CACHE_KEY_NAME, **{
            'id': self.id,
            'setting': RSTVIEW_PARSER_FILTER_SETTINGS.keys(),
            'header_level': ['None']+range(1, 7),
        })
        keys += _get_cache_keyset(INSERT_TOC_CACHE_KEY_NAME, **{
            'id': self.id,
            'setting': RSTVIEW_PARSER_FILTER_SETTINGS.keys(),
            'header_level': ['None']+range(1, 7),
        })
        cache.delete_many(keys)
        return keys
    
    class Meta:
        verbose_name = _("insert document")
        verbose_name_plural = _("insert document")

        
        
class PageModelBase(models.Model):
    """
    Model base for Pages
    """
    created = models.DateTimeField(_('created'), blank=True, editable=False)
    author = models.ForeignKey(User, verbose_name=_('author'))
    title = models.CharField(_('title'), blank=False, max_length=255)
    published = models.DateTimeField(_('publish date'), blank=True, help_text=_("Define when the document will be displayed on the site. Empty value mean an instant publish, use a coming date to program a futur publish."))
    template = models.CharField(_('template'), max_length=50, choices=DOCUMENTS_PAGE_TEMPLATES_CHOICES, default=DOCUMENTS_PAGE_TEMPLATE_DEFAULT, help_text=_("This template will be used to render the page."))
    order = models.SmallIntegerField(_('order'), default=1, help_text=_("Display order in lists and trees."))
    visible = models.BooleanField(_('visibility'), choices=DOCUMENTS_VISIBILTY_CHOICES, default=True)
    content = models.TextField(_('content'), blank=False)
    comment = models.CharField(_('comment'), blank=True, null=True, max_length=255)

    def __unicode__(self):
        return self.title

    def get_template(self):
        return DOCUMENTS_PAGE_TEMPLATES[self.template][0]
    
    class Meta:
        abstract = True    



class Page(PageModelBase):
    """
    Full page document
    """
    modified = models.DateTimeField(_('last edit'), auto_now=True)
    slug = models.SlugField(_('slug'), unique=True, max_length=75, help_text=_("Unique slug used in URL, should be automatically filled with sluggified title."))
    
    @models.permalink
    def get_absolute_url(self):
        return ('documents-page-details', [self.slug])
    
    def get_render_cache_key(self, **kwargs):
        """
        Get the cache key for the content render with according to the given settings
        """
        return PAGE_RENDER_CACHE_KEY_NAME.format(id=self.id, **kwargs)
    
    def get_toc_cache_key(self, **kwargs):
        """
        Get the cache key for the content TOC with according to the given settings
        """
        return PAGE_TOC_CACHE_KEY_NAME.format(id=self.id, **kwargs)
    
    def clear_cache(self):
        """
        Invalidate all possible cache keys
        """
        keys = _get_cache_keyset(PAGE_RENDER_CACHE_KEY_NAME, **{
            'id': self.id,
            'setting': RSTVIEW_PARSER_FILTER_SETTINGS.keys(),
        })
        keys += _get_cache_keyset(PAGE_TOC_CACHE_KEY_NAME, **{
            'id': self.id,
            'setting': RSTVIEW_PARSER_FILTER_SETTINGS.keys(),
        })
        # Drop cache for knowed pages slugs used in the ``page`` rest role
        cache.delete_many([PAGE_SLUGS_CACHE_KEY_NAME]+keys)
        return keys
    
    def _get_current_revision(self):
        return (self.revision.all().aggregate(models.Max('revision')).get('revision__max') or 0)+1
    current_revision = property(_get_current_revision)
    
    def save(self, *args, **kwargs):
        # First create
        if not self.created:
            self.created = datetime.now()
        # Creating a new revision archive
        elif DOCUMENTS_PAGE_ARCHIVED:
            old = Page.objects.get(pk=self.id)
            PageRevision(
                page=self,
                created=self.modified,
                parent=old.parent,
                author=old.author,
                title=old.title,
                published=old.published,
                slug=old.slug,
                template=old.template,
                order=old.order,
                visible=old.visible,
                content=old.content,
                revision=self.current_revision,
                comment=old.comment,
            ).save()
        
        # Fill in the published date with the created date if empty
        if not self.published:
            self.published = self.created
        # Invalidate all caches on edit
        if self.modified:
            self.clear_cache()
        
        super(Page, self).save(*args, **kwargs)
    
    def delete(self, using=None):
        self.clear_cache()
        super(Page, self).delete(using=using)
    
    class Meta:
        verbose_name = _("page")
        verbose_name_plural = _("pages")

# Add a parent foreign key to the model without sublassing MPTTModel
TreeForeignKey(Page, blank=True, null=True, related_name="%(app_label)s_%(class)s_children").contribute_to_class(Page, 'parent')
mptt.register(Page, order_insertion_by=['order', 'title'])



class PageRevision(PageModelBase):
    """
    History revision for a Page document
    """
    slug = models.SlugField(_('slug'), max_length=75)
    page = models.ForeignKey(Page, verbose_name=_('page'), related_name='revision')
    parent = models.ForeignKey(Page, null=True, blank=True, related_name="revision_parent")
    revision = models.IntegerField(_('revision number'), blank=False, default=0, editable=False)
    
    class Meta:
        verbose_name = _("page revision")
        verbose_name_plural = _("pages revisions")



class Attachment(models.Model):
    """
    Attachment file for a Page document
    
    TODO: Remove attachment map cache on update/delete and also when Page is removed, and updated ?
    """
    page = models.ForeignKey(Page, verbose_name=_('page'), related_name='attachment')
    author = models.ForeignKey(User, verbose_name=_('author'))
    title = models.CharField(_('title'), max_length=75, blank=True)
    slug = models.CharField(_('slug'), max_length=75, blank=True, help_text=_("Used as the key to put the attachment in your documents. This does not really use the slug syntax, you can use more special characters. If empty, will be filled with the original file name."))
    created = models.DateTimeField(_('created'), auto_now_add=True)
    file = models.FileField(_('file'), upload_to=ATTACH_FILE_UPLOADTO, max_length=255, blank=False)
    size = models.IntegerField(_('size'), blank=False, default=0, editable=False)
    content_type = models.CharField(_('content_type'), max_length=120, blank=False, null=True, editable=False)
    
    def __unicode__(self):
        return self.slug
    
    def is_image(self):
        return (self.content_type and self.content_type in IMAGE_MIMETYPES)
    
    def save(self, *args, **kwargs):
        if not self.slug:
            # TODO: slugify
            self.slug = self.file.name.strip()
        if not self.title:
            self.title = self.slug
        super(Attachment, self).save(*args, **kwargs)

    class Meta:
        verbose_name = _("attachment file")
        verbose_name_plural = _("attachment files")


# Declaring signals
documents_page_update_signal = django.dispatch.Signal(providing_args=["page_instance","edited"])
documents_insert_update_signal = django.dispatch.Signal(providing_args=["insert_instance","edited"])

## Signal receiver demo
#def documents_page_update_receiver(sender, **kwargs):
    #page_instance = kwargs['page_instance']
    #edited = kwargs['edited']
    
    #if edited:
        #print u"* {0} has been edited !".format(page_instance.title)
    #else:
        #print u"* {0} has been created !".format(page_instance.title)

## Connecting signal to the receiver
#documents_page_update_signal.connect(documents_page_update_receiver)



import datetime, json

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.views.generic.base import TemplateResponseMixin, View
from django.views.generic.list import BaseListView
from django.views.generic.edit import BaseDeleteView, FormMixin

from braces.views import JSONResponseMixin

class DownloadMixin(object):
    """
    Simple Mixin to send a downloadable content
    
    Inherits must have :
    
    * Filled the ``self.content_type`` attribute with the content content_type to send;
    * Implementation of ``get_filename()`` that return the filename to use in response 
      headers;
    * Implementation of ``get_content()`` that return the content to send as downloadable.
    
    If the content is a not a string, it is assumed to be a fileobject to send as 
    the content with its ``read()`` method.
    
    Optionnaly implement a ``close_content()`` to close specifics objects linked to 
    content fileobject, if it does not exists a try will be made on a close() method 
    on the content fileobject;
    
    A "get_filename_timestamp" method is implemented to return a timestamp to use in your 
    filename if needed, his date format is defined in "timestamp_format" attribute (in a 
    suitable way to use with strftime on a datetime object).
    """
    content_type = None
    timestamp_format = "%Y-%m-%d"
    
    def get_filename_timestamp(self):
        return datetime.datetime.now().strftime(self.timestamp_format)
    
    def get_filename(self, context):
        raise ImproperlyConfigured("DownloadMixin requires an implementation of 'get_filename()' to return the filename to use in headers")
    
    def get_content(self, context):
        raise ImproperlyConfigured("DownloadMixin requires an implementation of 'get_content()' to return the downloadable content")
    
    def render_to_response(self, context, **response_kwargs):
        if getattr(self, 'content_type', None) is None:
            raise ImproperlyConfigured("DownloadMixin requires a definition of 'content_type' attribute")
        # Needed headers
        response = HttpResponse(content_type=self.content_type, **response_kwargs)
        response['Content-Disposition'] = 'attachment; filename={0}'.format(self.get_filename(context))
        # Read the content file object or string, append it to response and close it
        content = self.get_content(context)
        if isinstance(content, basestring):
            response.write(content)
        else:
            response.write(content.read())
        # Conditionnal closing content object
        if hasattr(self, 'close_content'):
            self.close_content(context, content)
        elif hasattr(content, 'close'):
            content.close()
            
        return response

    def get_context_data(self, **kwargs):
        return {
            'params': kwargs
        }

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)


class ExtendTemplateVariableMixin(object):
    """
    Get the extend variable to use in the template
    
    Default behaviour is to switch on two templates depending on the request is an ajax 
    request or not, if ajax "base_modal.html" is used else the default extend will 
    simply be "base.html".
    
    This only put the "template_extend" variable in the template context, your template 
    have to use it, this does not modify itself the response nor the template.
    """
    default_extend_template = "base.html"
    modal_extend_template = "base_modal.html"
    
    def get_template_extend(self):
        if self.request.is_ajax():
            return self.modal_extend_template
        return self.default_extend_template
    
    def get_context_data(self, **kwargs):
        context = super(ExtendTemplateVariableMixin, self).get_context_data(**kwargs)
        context.update({
            'template_extend': self.get_template_extend(),
        })
        return context


class SimpleListView(TemplateResponseMixin, BaseListView):
    """
    Like generic.ListView but use only ``get_template`` to find template and not an 
    automatic process on ``get_template_names``
    """
    pass


class DirectDeleteView(BaseDeleteView):
    """
    To directly delete an object without template rendering on GET or POST methods
    
    "get_success_url" or "success_url" should be correctly filled
    """
    memoize_old_object = False
    _memoized_attr = ['id', 'slug', 'title'] # memoized attributes if exists
    old_object = {} # where the attributes will be putted
    
    def get(self, *args, **kwargs):
        if self.memoize_old_object:
            self.memoize_object()
        return self.delete(*args, **kwargs)
    
    def memoize_object(self, *args, **kwargs):
        old = self.get_object()
        for item in self._memoized_attr:
            self.old_object[item] = getattr(old, item, None)


class ListAppendView(SimpleListView, FormMixin):
    """
    A view to display an object list with a form to append a new object
    
    This view re-use some code from FormMixin and SimpleListView, sadly it seem not 
    possible to simply mix them.
    
    Need "model" and "form_class" attributes for the form parts and the required one 
    by BaseListView. "get_success_url" method should be filled too.
    
    "locked_form" is used to disable form (like if your list object is closed to new 
    object)
    """
    model = None
    form_class = None
    template_name = None
    paginate_by = None
    locked_form = False
    
    def form_valid(self, form):
        self.object = form.save()
        return super(ListAppendView, self).form_valid(form)

    def form_invalid(self, form):
        return self.render_to_response(self.get_context_data(object_list=self.object_list, form=form))

    def is_locked_form(self):
        return self.locked_form

    def get_form(self, form_class):
        """
        Returns an instance of the form to be used in this view.
        """
        if self.is_locked_form():
            return None
        return form_class(**self.get_form_kwargs())
        
    def get(self, request, *args, **kwargs):
        self.object_list = self.get_queryset()
        
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        
        allow_empty = self.get_allow_empty()
        if not allow_empty and len(self.object_list) == 0:
            raise Http404(_(u"Empty list and '%(class_name)s.allow_empty' is False.")
                          % {'class_name': self.__class__.__name__})
        
        context = self.get_context_data(object_list=self.object_list, form=form)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        self.object_list = self.get_queryset()
        
        form_class = self.get_form_class()
        form = self.get_form(form_class)
        
        allow_empty = self.get_allow_empty()
        if not allow_empty and len(self.object_list) == 0:
            raise Http404(_(u"Empty list and '%(class_name)s.allow_empty' is False.")
                          % {'class_name': self.__class__.__name__})
        
        if form and form.is_valid():
            return self.form_valid(form)
        elif form:
            return self.form_invalid(form)
        else:
            context = self.get_context_data(object_list=self.object_list, form=form)
            return self.render_to_response(context)

    def put(self, *args, **kwargs):
        return self.post(*args, **kwargs)

class DetailListAppendView(ListAppendView):
    """
    A view to display a parent object details, list his "children" and display a form 
    to append a new child
    
    Have the same behaviours than "ListAppendView" but get the parent object before 
    doing anything.
    
    "model" and "form_class" attribute are for the children, "context_parent_object_name" 
    is used to name the parent variable in the template context.
    
    "get_parent_object" must be defined to return the parent instance and also "get_queryset" 
    to filter queryset only for the children of the parent.
    
    The parent object is also given to the append form, under the name defined with the 
    "context_parent_object_name" attribute. Your Form should be aware of this.
    """
    context_parent_object_name = 'parent_object'
    
    def get_parent_object(self):
        raise ImproperlyConfigured(u"%(cls)s's 'get_parent_object' method must be defined " % {"cls": self.__class__.__name__})

    def get_context_data(self, **kwargs):
        kwargs.update({
            self.context_parent_object_name: self.parent_object,
        })
        return super(DetailListAppendView, self).get_context_data(**kwargs)
        
    def get_form_kwargs(self):
        """
        Returns an instance of the form to be used in this view.
        """
        kwargs = super(DetailListAppendView, self).get_form_kwargs()
        kwargs.update({
            self.context_parent_object_name: self.parent_object,
        })
        return kwargs
        
    def get(self, request, *args, **kwargs):
        self.parent_object = self.get_parent_object()
        return super(DetailListAppendView, self).get(request, *args, **kwargs)
        
    def post(self, request, *args, **kwargs):
        self.parent_object = self.get_parent_object()
        return super(DetailListAppendView, self).post(request, *args, **kwargs)

class JSONResponseExtendedMixin(JSONResponseMixin):
    """
    Simple Mixin to compile the context view as JSON
    
    This does not implement a direct response, you have to return it with the 
    ``json_to_response`` method in your view.
    """
    json_indent = None
    json_encoder = DjangoJSONEncoder
    json_ensure_ascii = False
    
    def encode_context(self, context):
        json_kwargs = {}
        if self.json_indent is not None:
            json_kwargs['indent'] = self.json_indent
        if self.json_encoder is not None:
            json_kwargs['encoder'] = self.json_encoder
        if self.json_ensure_ascii is not None:
            json_kwargs['ensure_ascii'] = self.json_ensure_ascii
        return json.dumps(context, **json_kwargs)

    def render_json_response(self, context_dict, **response_kwargs):
        """
        Limited serialization for shipping plain data. Do not use for models
        or other complex or custom objects.
        """
        if 'content_type' not in response_kwargs:
            response_kwargs['content_type'] = self.get_content_type()
        return HttpResponse(self.encode_context(context), **response_kwargs)


class JSONResponseViewMixin(JSONResponseExtendedMixin):
    """Mixin to directly return a JSON response"""
    def render_to_response(self, context, **response_kwargs):
        """
        Returns a response with a template rendered with the given context.
        """
        return self.json_to_response(context)



# -*- coding: utf-8 -*-
"""
Page document views
"""
import json, os
from datetime import datetime
from cStringIO import StringIO

from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404
from django.template.loader import render_to_string
from django.views import generic
from django.views.generic.detail import SingleObjectMixin

from mptt.templatetags.mptt_tags import cache_tree_children

from braces.views import LoginRequiredMixin, PermissionRequiredMixin

from guardian.mixins import PermissionRequiredMixin as PerObjectPermissionRequiredMixin

from djangocodemirror.views import SampleQuicksaveMixin

from rstview.parser import SourceParser

from sveedocuments import local_settings
from sveedocuments.models import Page, Attachment
from sveedocuments.forms.page import PageForm, PageEditForm, PageQuickForm
from sveedocuments.forms.attachment import AttachmentForm
from sveedocuments.utils.objects import get_instance_children
from sveedocuments.mixins import PageChangePermissionRequiredMixin
from sveedocuments.utils.braces_addons import DetailListAppendView, DirectDeleteView, DownloadMixin

class PageIndexView(generic.TemplateView):
    """
    Pages index
    """
    template_name = "sveedocuments/index.html"
    
    def get(self, request, *args, **kwargs):
        # Force queryset interpretation to list so mptt recursing does not 
        # trigger two request from the same queryset
        page_list = list(Page.objects.filter(visible=True))
        
        context = {
            'page_list': page_list,
            'json_tree': [],
        }
        
        # Fill the JSON tree if there is at least one entry
        if page_list:
            context['json_tree'] = json.dumps(self.get_recursed_tree( cache_tree_children(page_list) ), indent=4)
            
        return self.render_to_response(context)
    
    def get_recursed_tree(self, root_nodes):
        """
        Get the recursed tree from the queryset
        """
        nodes = []
        for n in root_nodes:
            nodes.append({
                "id": n.pk,
                "label": n.title,
                "slug": n.slug,
                "children": self.get_recursed_tree(n.get_children())
            })
        return nodes

class HelpPageView(generic.TemplateView):
    """
    Help document
    """
    template_name = "sveedocuments/help.html"
    
    def get(self, request, *args, **kwargs):
        path_root = os.path.abspath(os.path.dirname(local_settings.__file__))
        f = open(os.path.join(path_root, "HELP.rst"))
        content = f.read()
        f.close()
        
        context = {'content' : SourceParser(content, silent=False)}
        return self.render_to_response(context)

class PageDetailsMixin(object):
    """
    *Page* view
    """
    model = Page
    context_object_name = "page_instance"
    template_name = "sveedocuments/page_details/page_default.html"
    
    def get_object(self, *args, **kwargs):
        """
        Memorize object to avoid multiple database access when using ``get_object()`` 
        method
        """
        cache_key = "_cache_get_object"
        if not hasattr(self, cache_key):
            setattr(self, cache_key, super(PageDetailsMixin, self).get_object(*args, **kwargs))
        return getattr(self, cache_key)
    
    def get_attachments(self):
        return self.object.attachment.all()
        
    def get_context_data(self, **kwargs):
        context = super(PageDetailsMixin, self).get_context_data(**kwargs)
        context.update({
            'attachments': self.get_attachments(),
        })
        return context
    
    def get(self, request, **kwargs):
        # Check if the object is ``visible``
        if not self.get_object().visible:
            raise Http404
        return super(PageDetailsMixin, self).get(request, **kwargs)
    
    def get_template_names(self):
        return [self.object.get_template()]

if local_settings.DOCUMENTS_PAGE_RESTRICTED:
    class PageDetailsView(PageDetailsMixin, LoginRequiredMixin, generic.DetailView):
        pass
else:
    class PageDetailsView(PageDetailsMixin, generic.DetailView):
        pass

class PageSourceView(PageDetailsView):
    """
    Raw content *Page* view
    """
    def get(self, request, *args, **kwargs):
        # Check if the object is ``visible``
        if not self.get_object().visible:
            raise Http404
        return HttpResponse(self.get_object().content, content_type="text/plain; charset=utf-8")

class PageCreateView(PermissionRequiredMixin, generic.CreateView):
    """
    Form view to create a *Page* document
    """
    model = Page
    context_object_name = "page_instance"
    template_name = "sveedocuments/board/page_form.html"
    form_class = PageForm
    permission_required = "sveedocuments.add_page"
    raise_exception = True
    _redirect_to_self = False

    def get(self, request, *args, **kwargs):
        self.parent_page_instance = self._get_parent(**kwargs)
        return super(PageCreateView, self).get(request, *args, **kwargs)
        
    def post(self, request, *args, **kwargs):
        self.parent_page_instance = self._get_parent(**kwargs)
        # Mark to go back to the form page after save, triggered by special submit
        if request.POST:
            if request.POST.get('submit_and_continue', False):
                self._redirect_to_self = True
        return super(PageCreateView, self).post(request, *args, **kwargs)
        
    def get_success_url(self):
        if self._redirect_to_self:
            return reverse('sveedocuments:page-edit', args=[self.object.slug])
        return reverse('sveedocuments:page-index')

    def _get_parent(self, **kwargs):
        if 'slug' in kwargs:
            return get_object_or_404(Page, slug=kwargs['slug'])
        return None
        
    def get_context_data(self, **kwargs):
        context = super(PageCreateView, self).get_context_data(**kwargs)
        context.update({
            'parent_page_instance': self.parent_page_instance,
        })
        return context
    
    def get_form_kwargs(self):
        kwargs = super(PageCreateView, self).get_form_kwargs()
        kwargs.update({
            'author': self.request.user,
            'parent': self.parent_page_instance,
            'initial': {'published': datetime.today()},
        })
        return kwargs

class PageTabsContentMixin(object):
    def get_context_data(self, **kwargs):
        context = super(PageTabsContentMixin, self).get_context_data(**kwargs)
        # Little trick to work with SingleObjectMixin and DetailListAppendView
        if hasattr(self, 'parent_object'):
            obj = self.parent_object
        else:
            obj = self.object
        
        context.update({
            'revisions_count': obj.revision.all().count()+1,
            'files_count': obj.attachment.all().count(),
        })
        return context

class PageEditView(PerObjectPermissionRequiredMixin, PageTabsContentMixin, generic.UpdateView):
    """
    Form view to edit a *Page* document
    """
    model = Page
    context_object_name = "page_instance"
    template_name = "sveedocuments/board/page_form.html"
    form_class = PageEditForm
    permission_required = "sveedocuments.change_page"
    accept_global_perms = True
    raise_exception = True
    _redirect_to_self = False
        
    def post(self, request, *args, **kwargs):
        # Mark to go back to the form page after save, triggered by special submit
        if request.POST:
            if request.POST.get('submit_and_continue', False):
                self._redirect_to_self = True
        return super(PageEditView, self).post(request, *args, **kwargs)
    
    def get_object(self, *args, **kwargs):
        """
        Forcing a new empty comment field in edit mode
        """
        obj = super(PageEditView, self).get_object(*args, **kwargs)
        obj.comment = ''
        return obj

    def get_success_url(self):
        if self._redirect_to_self:
            return reverse('sveedocuments:page-edit', args=[self.object.slug])
        return reverse('sveedocuments:page-index')

    def get_form_kwargs(self):
        kwargs = super(PageEditView, self).get_form_kwargs()
        kwargs.update({'author': self.request.user})
        return kwargs


class PageHistoryView(PerObjectPermissionRequiredMixin, PageTabsContentMixin, generic.DetailView):
    """
    *Page* history
    """
    model = Page
    context_object_name = "page_instance"
    template_name = "sveedocuments/board/page_history.html"
    permission_required = "sveedocuments.change_page"
    accept_global_perms = True
    raise_exception = True
    
    def get_object(self, *args, **kwargs):
        """
        Memorize object to avoid multiple database access when using ``get_object()`` 
        method
        """
        cache_key = "_cache_get_object"
        if not hasattr(self, cache_key):
            setattr(self, cache_key, super(PageHistoryView, self).get_object(*args, **kwargs))
        return getattr(self, cache_key)
        
    def get_context_data(self, **kwargs):
        context = super(PageHistoryView, self).get_context_data(**kwargs)
        context.update({
            'last_revisions': self.object.revision.all().order_by('-created'),
        })
        return context


class PageDeleteView(PerObjectPermissionRequiredMixin, generic.DeleteView):
    """
    Form view to delete a *Page* document
    
    This display a tree of relations to confirm the object and relations deletion
    """
    model = Page
    context_object_name = "page_instance"
    template_name = "sveedocuments/board/page_delete.html"
    permission_required = "sveedocuments.delete_page"
    accept_global_perms = True
    raise_exception = True

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        context.update({'relations': get_instance_children(self.object)})
        return self.render_to_response(context)

    def get_success_url(self):
        return reverse('sveedocuments:page-index')


class PageAttachmentsView(PageChangePermissionRequiredMixin, PageTabsContentMixin, DetailListAppendView):
    """
    Form view to add file attachments to a Page
    """
    model = Attachment
    form_class = AttachmentForm
    template_name = "sveedocuments/board/page_attachments.html"
    raise_exception = True
    accept_global_perms = True
    context_parent_object_name = 'page_instance'
    
    def get_page_object(self):
        return self.get_parent_object()
    
    def get_parent_object(self):
        return get_object_or_404(Page, slug=self.kwargs['slug'])
    
    def get_queryset(self):
        return self.parent_object.attachment.all()

    def get_success_url(self):
        return reverse('sveedocuments:page-attachments', args=[self.parent_object.slug])

    def get_form_kwargs(self):
        kwargs = super(PageAttachmentsView, self).get_form_kwargs()
        kwargs.update({'author': self.request.user})
        return kwargs


class PageAttachmentDeleteView(PageChangePermissionRequiredMixin, DirectDeleteView):
    """
    View to delete a *Page* document
    """
    model = Attachment
    raise_exception = True
    memoize_old_object = True
    accept_global_perms = True
    _memoized_attr = ['id', 'slug', 'title', 'page']
    
    def get_page_object(self):
        return get_object_or_404(Page, slug=self.kwargs['slug'])

    def get_success_url(self):
        return reverse('sveedocuments:page-attachments', args=[self.old_object['page'].slug])


class PageQuicksaveView(SampleQuicksaveMixin, PageEditView):
    """
    Quicksave view for a *Page* content
    """
    form_class = PageQuickForm
    
    def get_object(self, queryset=None):
        if self.request.POST.get('slug', False):
            self.kwargs['slug'] = self.request.POST['slug']
        return super(PageQuicksaveView, self).get_object(queryset=queryset)

# Optional view for PDF export need RstToPdf
try:
    from rst2pdf.createpdf import RstToPdf
except ImportError:
    class PagePDFView(object):
        """Dummy object"""
        is_dummy = True
else:
    class PagePDFView(SingleObjectMixin, DownloadMixin, generic.View):
        """
        Export a Page as a PDF file
        """
        model = Page
        content_type = 'application/pdf'
        filename_format = "{slug}_{timestamp}.pdf"
        
        def get_context_data(self, **kwargs):
            context = super(PagePDFView, self).get_context_data(**kwargs)
            context.update({
                'slug': self.object.slug,
                'timestamp': self.get_filename_timestamp(),
            })
            return context
        
        def get_filename(self, context):
            return self.filename_format.format(**context)
        
        def get_content(self, context):
            """
            Build the PDF from the page content with RstToPdf
            """
            # Add the optional preface and then the page content
            content = ''
            if getattr(local_settings, 'DOCUMENTS_EXPORT_PDF_PREFACE', None) is not None:
                content += local_settings.DOCUMENTS_EXPORT_PDF_PREFACE.format(page=self.object)
            content += self.object.content
            
            buffer_response = StringIO()
            # TODO: Can't we simply pass a dict kwargs to "RstToPdf" so all available 
            #       options could be setted from the settings ?
            RstToPdf(language=settings.LANGUAGE_CODE, breakside='any', header=local_settings.DOCUMENTS_EXPORT_PDF_HEADER, footer=local_settings.DOCUMENTS_EXPORT_PDF_FOOTER).createPdf(text=content, output=buffer_response)
            pdf = buffer_response.getvalue()
            buffer_response.close()
            
            return pdf

        def get(self, request, *args, **kwargs):
            self.object = self.get_object()
            context = self.get_context_data(**kwargs)
            return super(PagePDFView, self).render_to_response(context)



# -*- coding: utf-8 -*-
def get_instance_children(obj, depth=0, sig=0):
    """
    Récupèration récursive des relations enfants d'un objet
    
    @depth: integer limitant le niveau de recherche des enfants, 0=illimité
    """
    children = []
    # Pour toute les relations enfants de l'objet
    for child in obj._meta.get_all_related_objects():
        # Nom de l'attribut d'accès
        cname = child.get_accessor_name()
        verbose_name = child.model._meta.verbose_name
        # Récupère tout les objets des relations
        for elem in getattr(obj, cname).all():
            followed = []
            # Recherche récursive des enfants
            if depth == 0 or sig < depth:
                followed = get_instance_children(elem, depth=depth, sig=sig+1)
            children.append( (verbose_name, unicode(elem), followed) )
    return children



# -*- coding: utf-8 -*-
"""
Application Crumbs
"""
from autobreadcrumbs import site
from django.utils.translation import ugettext_lazy

site.update({
    'sveedocuments:index': ugettext_lazy('Document sitemap'),
    'sveedocuments:board': ugettext_lazy('Document management'),
    'sveedocuments:insert-index': ugettext_lazy('Insert documents'),
    'sveedocuments:insert-add': ugettext_lazy('Add a new insert document'),
    'sveedocuments:insert-delete': ugettext_lazy('Delete #{{ insert_instance.slug }}'),
    'sveedocuments:insert-edit': ugettext_lazy('Edit #{{ insert_instance.slug }}'),
    'sveedocuments:page-index': ugettext_lazy('Pages'),
    'sveedocuments:page-add': ugettext_lazy('Add a new page'),
    'sveedocuments:page-details': ugettext_lazy('{{ page_instance.title }}'),
    'sveedocuments:page-delete': ugettext_lazy('Delete #{{ page_instance.slug }}'),
    'sveedocuments:page-history': ugettext_lazy('History'),
    'sveedocuments:page-attachments': ugettext_lazy('Attachments'),
    'sveedocuments:help': ugettext_lazy('Usage help'),
    'sveedocuments:page-edit': ugettext_lazy('Edit #{{ page_instance.slug }}'),
    'sveedocuments:page-add-child': ugettext_lazy('Add a new child page to #{{ parent_page_instance.slug }}'),
})



# -*- coding: utf-8 -*-
"""
Template tags used in admin or board
"""
from django import template

register = template.Library()

@register.filter(name='calcul_indent')
def calcul_indent(value, coef=20):
    return (value+1)*coef
